# Copyright (c) 2016, 2018, Oracle and/or its affiliates. All rights reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2.0, as
# published by the Free Software Foundation.
#
# This program is also distributed with certain software (including
# but not limited to OpenSSL) that is licensed under separate terms,
# as designated in a particular file or component or in included license
# documentation.  The authors of MySQL hereby grant you an
# additional permission to link the program and your derivative works
# with the separately licensed software that they have included with
# MySQL.
#
# Without limiting anything contained in the foregoing, this file,
# which is part of MySQL Connector/Python, is also subject to the
# Universal FOSS Exception, version 1.0, a copy of which can be found at
# http://oss.oracle.com/licenses/universal-foss-exception.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License, version 2.0, for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA

"""Constants."""

from collections import namedtuple


# pylint: disable=C0103
def create_enum(name, fields, values=None):
    """Emulates an enum by creating a namedtuple.

    Args:
        name (string): The type name.
        fields (tuple): The fields names.
        values (tuple): The values of the fields.

    Returns:
        namedtuple: A namedtuple object.
    """
    Enum = namedtuple(name, fields)
    if values is None:
        return Enum(*fields)
    return Enum(*values)


SSLMode = create_enum("SSLMode",
                      ("REQUIRED", "DISABLED", "VERIFY_CA", "VERIFY_IDENTITY"),
                      ("required", "disabled", "verify_ca", "verify_identity"))
Auth = create_enum("Auth",
                   ("PLAIN", "MYSQL41", "SHA256_MEMORY"),
                   ("plain", "mysql41", "sha256_memory"))
LockContention = create_enum("LockContention",
                             ("DEFAULT", "NOWAIT", "SKIP_LOCKED"), (0, 1, 2))

__all__ = ["SSLMode", "Auth", "LockContention"]
